/****************************************************************************
**  DerBrowserTimer - Timer/Clock/Alarm add-on for Mozilla
**  Copyright (c) 2003  Charles Melhorn
**
**  This file is part of DerBrowserTimer.
**
**  DerBrowserTimer is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  DerBrowserTimer is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 

const TS_OFF      = 0;  // timer states
const TS_TIMING   = 1;
const TS_ALARMING = 2;

const AT_TIMER    = 0;  // alarm types
const AT_ALARM    = 1;
const AT_PERIODIC = 2;

const TP_QUARTERHR = 15; // target periods
const TP_HALFHR    = 30;
const TP_HOURLY    = 60;

const PT_BOOL         = 0;  // preference types
const PT_INT          = 1;
const PT_STR          = 2;
const PT_UNICHAR      = 3;
const PT_UNICHARLOCAL = 4;
const PT_FILE         = 5;

var gSelected_alarm_type = AT_TIMER;


function initAlarmSet()
{
  document.getElementById("periodic-options-radiogroup").disabled = "true";
}


function doOk()
{
  window.arguments[0].bAlarmSet = true;
  window.arguments[0].reminder_msg = document.getElementById('remindermsg-input').value;

  if (gSelected_alarm_type == AT_TIMER)
   {
     var countdownminutes = (document.getElementById('m-interval-tbox')).value;
     var countdownhours = (document.getElementById('h-interval-tbox')).value;
     //alert("value entered is "+countdownhours+":"+countdownminutes);

     if (countdownhours == "")
       countdownhoursInt = 0;
     else
       countdownhoursInt = parseInt(countdownhours);
 
     if (countdownminutes == "")
       countdownminutesInt = 0;
     else
       countdownminutesInt = parseInt(countdownminutes);

     if ( (countdownhoursInt < 0) || (countdownminutesInt < 0) )
      {
        alert("Error 01: Invalid Count Down Setting - please reenter");
        return false;
      }
     else
      {
        addseconds = (countdownhoursInt * 3600) + (countdownminutesInt * 60);
        addmillisecs = addseconds * 1000;
        today = new Date();
        window.arguments[0].targetDate = new Date();
        window.arguments[0].targetDate.setTime(today.getTime() + addmillisecs);
        window.arguments[0].countdown_time = (countdownhoursInt * 60) + countdownminutesInt;
        window.arguments[0].snooze_increment = Math.max(Math.ceil(0.1 * window.arguments[0].countdown_time), 1);
      }
   }
  else if (gSelected_alarm_type == AT_ALARM)
   {
     var target_hours = (document.getElementById('h-target-tbox')).value;
     var target_mins = (document.getElementById('m-target-tbox')).value;
     var timeofday_spec = (document.getElementById('target-timeofdayspec-listbox')).selectedItem.value;

     if (target_hours == "")
       target_hoursInt = 0;
     else
       target_hoursInt = parseInt(target_hours);

     if (target_mins == "")
       target_minsInt = 0;
     else
       target_minsInt = parseInt(target_mins);

     if ( (target_hoursInt < 0) || (target_hoursInt > 23) ||
          (target_minsInt < 0) || (target_minsInt > 59) ||
          ((timeofday_spec != "mil") && (target_hoursInt > 12)) )
      {
        alert("Error 02: Invalid Alarm Setting - please reenter");
        return false;
      }
     else
      {

        today = new Date();
        window.arguments[0].targetDate = new Date();

        if (timeofday_spec == "pm")
         {
           if (target_hoursInt < 12)
             target_hoursInt += 12;
         }
        else if (timeofday_spec == "am")
         {
           if (target_hoursInt == 12)
             target_hoursInt = 0;
         }

        //alert(gTargetdate);
        window.arguments[0].targetDate.setHours(target_hoursInt);
        window.arguments[0].targetDate.setMinutes(target_minsInt);
        window.arguments[0].targetDate.setSeconds(0);
        window.arguments[0].targetDate.setMilliseconds(0);

        // if current time >= alarm time, add 24hrs to alarm time

        if ((today.getTime()) >= (window.arguments[0].targetDate.getTime()) )
          window.arguments[0].targetDate.setTime(window.arguments[0].targetDate.getTime() + 24*60*60*1000);        

        //alert(gTargetdate);
        window.arguments[0].origTargetDate = new Date();
        window.arguments[0].origTargetDate.setTime(window.arguments[0].targetDate.getTime());
        window.arguments[0].snooze_increment = 10;
      }  
   }
  else // gSelected_alarm_type == AT_PERIODIC
   {
     var target_period_str = (document.getElementById('periodic-options-radiogroup')).selectedItem.value;
     var period_offset_str = (document.getElementById('period-offset-tbox')).value;

     var target_period = parseInt(target_period_str);
     var period_offset;

     if (period_offset_str == "")
       period_offset = 0;
     else
       period_offset = parseInt(period_offset_str);

     if (Math.abs(period_offset) > target_period)
      {
        alert("Error 03: Offset value too large - please reenter");
        return false;
      }
     else
      {
        if (period_offset < 0)
          period_offset = target_period + period_offset;

        //alert("target period: " + target_period + " offset: " + period_offset);
        today = new Date();
        window.arguments[0].targetDate = new Date();
        var current_minutes = window.arguments[0].targetDate.getMinutes();
        //alert("curr minutes: " + current_minutes);
        window.arguments[0].targetDate.setSeconds(0);
        window.arguments[0].targetDate.setMilliseconds(0);

        var mins_to_next_period = (target_period - (current_minutes % target_period));
        var adjusted_for_offset_mins = (mins_to_next_period + period_offset) % target_period;

        if (adjusted_for_offset_mins == 0)
          adjusted_for_offset_mins = target_period;

        var target_adjust_ms = adjusted_for_offset_mins*60*1000;

        window.arguments[0].targetDate.setTime(window.arguments[0].targetDate.getTime() + target_adjust_ms);
        window.arguments[0].snooze_increment = 5;
        window.arguments[0].target_period = target_period;
        window.arguments[0].period_offset = period_offset;
      }  
   }

  window.arguments[0].alarm_type = gSelected_alarm_type;

}


function plusTenButtonHandler()
{  
  var countdownhoursInt, countdownminutesInt, totalminutes;
  var countdownminutes_input = document.getElementById('m-interval-tbox');
  var countdownhours_input = document.getElementById('h-interval-tbox');

  if (gSelected_alarm_type == AT_TIMER)
   {
     var countdownminutes = countdownminutes_input.value;
     var countdownhours = countdownhours_input.value;
     
     if (countdownhours == "")
       countdownhoursInt = 0;
     else
       countdownhoursInt = parseInt(countdownhours);

     if (countdownminutes == "")
       countdownminutesInt = 0;
     else
       countdownminutesInt = parseInt(countdownminutes);

     if ( (countdownhoursInt >= 0) && (countdownminutesInt >= 0) )
      {
        totalminutes = countdownhoursInt*60 + countdownminutesInt;
 
        totalminutes += 10;
        putback_hours = Math.floor(totalminutes / 60);
        putback_minutes = totalminutes % 60

        countdownhours_input.value = ((putback_hours < 10) ? "0" : "") + putback_hours;
        countdownminutes_input.value = ((putback_minutes < 10) ? "0" : "") + putback_minutes;
      }
   }
}


function enablePeriodicRadioButtons(bEnable)
{
  var periodic_rgroup = document.getElementById('periodic-options-radiogroup');
  var hourly_offset_input = document.getElementById('period-offset-tbox');
  var h_offset_label = document.getElementById('hourly-offset-lbl');

  if (bEnable == true)
   { 
     periodic_rgroup.disabled = false;
     hourly_offset_input.disabled = false;
     h_offset_label.setAttribute("disabled", "false");
   }
  else
   {
     periodic_rgroup.disabled = true;
     hourly_offset_input.disabled = true;
     h_offset_label.setAttribute("disabled", "true");
   }

}


function SelectAlarmType(atype)
{
  gSelected_alarm_type = atype;

  switch (atype)
   {
     case AT_TIMER: 
       (document.getElementById('h-interval-tbox')).disabled = false;
       (document.getElementById('m-interval-tbox')).disabled = false;
       (document.getElementById('plus-ten-btn')).disabled = false;
       (document.getElementById('h-target-tbox')).disabled = true;
       (document.getElementById('m-target-tbox')).disabled = true;
       (document.getElementById('target-timeofdayspec-listbox')).disabled = true;
       enablePeriodicRadioButtons(false);
       break;

     case AT_ALARM:
       (document.getElementById('h-interval-tbox')).disabled = true;
       (document.getElementById('m-interval-tbox')).disabled = true;
       (document.getElementById('plus-ten-btn')).disabled = true;
       (document.getElementById('h-target-tbox')).disabled = false;
       (document.getElementById('m-target-tbox')).disabled = false;
       (document.getElementById('target-timeofdayspec-listbox')).disabled = false;
       enablePeriodicRadioButtons(false);
       break;

     case AT_PERIODIC:
       (document.getElementById('h-interval-tbox')).disabled = true;
       (document.getElementById('m-interval-tbox')).disabled = true;
       (document.getElementById('plus-ten-btn')).disabled = true;
       (document.getElementById('h-target-tbox')).disabled = true;
       (document.getElementById('m-target-tbox')).disabled = true;
       (document.getElementById('target-timeofdayspec-listbox')).disabled = true;
       enablePeriodicRadioButtons(true);
       break;

     default:
       alert("Error: Invalid alarm type");
   }

}


